"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var ElasticsearchService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElasticsearchService = void 0;
const common_1 = require("@nestjs/common");
const elasticsearch = require("@elastic/elasticsearch");
let ElasticsearchService = ElasticsearchService_1 = class ElasticsearchService {
    constructor(options) {
        this.options = options;
        this.logger = new common_1.Logger(ElasticsearchService_1.name);
        this.IElasticSearchConfig = options;
        this.esclient = new elasticsearch.Client({
            node: `${this.IElasticSearchConfig.node}:${this.IElasticSearchConfig.port}`,
        });
    }
    async createIndex(indexName, fields) {
        try {
            const fieldsList = fields.reduce((obj, item) => Object.assign(obj, { [item.fieldName]: { type: item.type } }), {});
            const existsStatus = await this.esclient.indices.exists({
                index: indexName,
            });
            if (existsStatus && existsStatus.statusCode === 200) {
                return this.esclient.indices.putMapping({
                    index: indexName,
                    body: {
                        properties: fieldsList,
                    },
                });
            }
            else {
                return this.esclient.indices.create({
                    index: indexName,
                    body: {
                        mappings: {
                            properties: fieldsList,
                        },
                    },
                });
            }
        }
        catch (e) {
            this.logger.error(e);
            throw e;
        }
    }
    async bulkInsert(docs, index, type) {
        const bulk = [];
        docs.forEach((doc) => {
            bulk.push({
                index: { _index: index, _type: type },
            });
            bulk.push(doc);
        });
        try {
            return this.esclient.bulk({
                body: bulk,
                index: index,
                type: type,
            });
        }
        catch (e) {
            this.logger.error(e);
            throw e;
        }
    }
    async searchIndex(q, index, skip, limit, queryType, fields) {
        const query = this.generateQuery(q, queryType, fields);
        const body = {
            size: limit,
            from: skip,
            query: query,
        };
        try {
            return this.esclient.search({ index: index, body });
        }
        catch (e) {
            this.logger.error(e);
            throw e;
        }
    }
    async updateIndex(index, query, updatetedFields) {
        let preparedUpdatedFields = '';
        for (const property in updatetedFields) {
            preparedUpdatedFields += `ctx._source["${property}"] = "${updatetedFields[property]}";`;
        }
        try {
            return this.esclient.updateByQuery({
                index: index,
                refresh: true,
                body: {
                    query: query,
                    script: {
                        lang: 'painless',
                        source: preparedUpdatedFields,
                    },
                },
            });
        }
        catch (e) {
            this.logger.error(e);
            throw e;
        }
    }
    async removeDocumentFromIndex(indexName, query) {
        try {
            return this.esclient.deleteByQuery({
                index: indexName,
                body: {
                    query: query,
                },
            });
        }
        catch (e) {
            this.logger.error(e);
            throw e;
        }
    }
    async reindex(indexFrom, indexDest, query) {
        try {
            return this.esclient.reindex({
                refresh: true,
                body: {
                    source: {
                        index: indexFrom,
                        query: query,
                    },
                    dest: {
                        index: indexDest,
                    },
                },
            });
        }
        catch (e) {
            this.logger.error(e);
            throw e;
        }
    }
    generateQuery(q, queryType, fields) {
        if (queryType === 'SimpleQuery') {
            return {
                multi_match: {
                    query: q,
                    type: 'phrase',
                    fields: fields,
                },
            };
        }
        return {};
    }
};
exports.ElasticsearchService = ElasticsearchService;
exports.ElasticsearchService = ElasticsearchService = ElasticsearchService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)('ELASTICSEARCH_CONFIG')),
    __metadata("design:paramtypes", [Object])
], ElasticsearchService);
//# sourceMappingURL=elasticsearch.service.js.map