export interface IDoc {
    id: string;
}
export declare enum SortDirection {
    ASC = "asc",
    DESC = "desc"
}
export type ESSort<T> = {
    [K in keyof Partial<T>]: SortDirection;
};
export interface ESFilterReqBody {
    index: string;
    type: string;
    analyzeWildcard?: boolean;
    body: {
        query?: any;
    };
    size?: number;
    from?: number;
    sort?: string[];
}
export type ESObjectFilter<T> = {
    [K in keyof Partial<T>]: string;
};
export type ElasticsearchFindResponse<T extends IDoc> = {
    hits: {
        hits: {
            _id: string;
            _source: T;
            _score: number;
        }[];
        total: {
            value: number;
        };
    };
};
export type ESFindResponse<T extends IDoc, D extends BaseDocument<T>> = {
    items: D[];
    total: number;
};
export type ESFilter<T> = ESObjectFilter<T> | string;
export declare class BaseDocument<T extends IDoc> {
    id: string;
    constructor(data?: Partial<T>);
    get(): Promise<BaseDocument<T>>;
    findById(id: string): Promise<BaseDocument<T>>;
    save(): Promise<BaseDocument<T>>;
    update(): Promise<BaseDocument<T>>;
    partialUpdate(data: Partial<T>): Promise<BaseDocument<T>>;
    remove(): Promise<boolean>;
    find<D extends BaseDocument<T>>(params: ESFilter<T>, from?: number, size?: number, sort?: {
        [key: string]: SortDirection;
    }): Promise<ESFindResponse<T, D>>;
    removeWhereNotIn(term: string, array: string[]): Promise<number>;
    private generateSort;
    private generateQuery;
    private apply;
    toObject(): T;
    toJson(): T;
}
