"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var IndexManagerService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndexManagerService = void 0;
const common_1 = require("@nestjs/common");
const elasticsearch = require("@elastic/elasticsearch");
const index_name_helper_1 = require("../../helpers/index-name.helper");
let IndexManagerService = IndexManagerService_1 = class IndexManagerService {
    constructor(options) {
        this.options = options;
        this.logger = new common_1.Logger(IndexManagerService_1.name);
        this.esclient = new elasticsearch.Client({
            node: `${this.options.node}:${this.options.port}`,
            auth: this.options.auth,
        });
    }
    async initialize() {
        try {
            let settings = {};
            const { body: { version }, } = await this.esclient.info();
            if (this.options.models) {
                const models = this.options.models;
                for (const model of models) {
                    const metadata = model.prototype;
                    model.prototype.client = this.esclient;
                    model.prototype.version = version.number;
                    model.prototype.logger = new common_1.Logger(model.name);
                    const indexName = (0, index_name_helper_1.getIndexName)(this.options.index, metadata.__type, version.number);
                    model.prototype.__indexName = indexName;
                    if (metadata.__type) {
                        if (!settings.hasOwnProperty(metadata.__type)) {
                            settings = Object.assign(Object.assign({}, settings), { [metadata.__type]: {} });
                        }
                        if (metadata.__fields) {
                            let fields = [];
                            Object.keys(metadata.__fields).forEach((f) => {
                                fields = [
                                    ...fields,
                                    Object.assign({ fieldName: f }, metadata.__fields[f]),
                                ];
                            });
                            await this.createIndex(indexName, metadata.__type, fields);
                        }
                    }
                }
            }
        }
        catch (e) {
            this.logger.error(e);
            throw e;
        }
    }
    async createIndex(indexName, type, fields) {
        var _a;
        try {
            let fieldsList = {};
            fields.forEach((_a) => {
                var { defaultValue, fieldName } = _a, config = __rest(_a, ["defaultValue", "fieldName"]);
                fieldsList = Object.assign(Object.assign({}, fieldsList), { [fieldName]: config });
            });
            const existsStatus = await this.esclient.indices.exists({
                index: indexName,
            });
            let index;
            if (!existsStatus || existsStatus.statusCode !== 200) {
                index = await this.esclient.indices.create({
                    index: indexName,
                    include_type_name: true,
                    body: {
                        settings: {
                            number_of_shards: 1,
                        },
                    },
                });
                this.logger.log(`Elasticsearch Index created: ${indexName}`);
            }
            else {
                index = await this.esclient.indices.get({
                    index: indexName,
                    include_type_name: true,
                    expand_wildcards: 'all',
                });
                this.logger.log(`Elasticsearch Index exists: ${indexName}`);
            }
            const mappings = (_a = index.body[indexName]) === null || _a === void 0 ? void 0 : _a.mappings;
            let skipMapping = true;
            const body = {
                properties: fieldsList,
            };
            if (!(mappings === null || mappings === void 0 ? void 0 : mappings[type])) {
                skipMapping = false;
            }
            if (Object.keys(body.properties).some((f) => {
                var _a, _b, _c, _d, _e;
                return !((_b = (_a = mappings === null || mappings === void 0 ? void 0 : mappings[type]) === null || _a === void 0 ? void 0 : _a.properties) === null || _b === void 0 ? void 0 : _b[f]) ||
                    body.properties[f].type != ((_e = (_d = (_c = mappings === null || mappings === void 0 ? void 0 : mappings[type]) === null || _c === void 0 ? void 0 : _c.properties) === null || _d === void 0 ? void 0 : _d[f]) === null || _e === void 0 ? void 0 : _e.type);
            })) {
                skipMapping = false;
            }
            if (!skipMapping) {
                this.logger.log(`Elasticsearch Mapping puted: ${indexName}/${type}`);
                index = await this.esclient.indices.putMapping({
                    index: indexName,
                    include_type_name: true,
                    type,
                    body,
                });
            }
            return index;
        }
        catch (e) {
            this.logger.error(e);
            throw e;
        }
    }
};
exports.IndexManagerService = IndexManagerService;
exports.IndexManagerService = IndexManagerService = IndexManagerService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)('ELASTICSEARCH_CONFIG')),
    __metadata("design:paramtypes", [Object])
], IndexManagerService);
//# sourceMappingURL=index-manager.service.js.map